==> v4_navigation_engine-0.1.30.tar.gz: ios/Classes/NavigationV4/View/MyNavigationRouteView+NavigationServiceDelegate.swift <==
//
//  MyNavigationRouteView+NavigationServiceDelegate.swift
//  NavigationUI
//
//  Created by hassine othmane on 4/2/20.
//  Copyright © 2020 com.mile. All rights reserved.
//

import MapboxCoreNavigation
import MapboxDirections
import CoreLocation
import MapboxNavigation


import MapboxSpeech
import AVKit

extension MyNavigationView :NavigationServiceDelegate
{
    
    public func navigationService(_ service: NavigationService, shouldRerouteFrom location: CLLocation) -> Bool {
        if service.route == wayToRoute
        {
            return true
        }
        return false
    }

    

    
    public func navigationService(_ service: NavigationService, didArriveAt waypoint: Waypoint) -> Bool {
        
        if wayToRoute != nil
        {
            stopNavigation()
            PlayerManager.shared.playSynt(description: self.languageRessource.routeReached)
            DispatchQueue.main.asyncAfter(deadline: DispatchTime.now()+2) {
                self.initNavigation(userRoute: self.myRoute!)
            }
        }
        else
        {
            // you must check waypoints returned by calculate route
            let loc1 = CLLocation(latitude: waypoint.coordinate.latitude, longitude: waypoint.coordinate.longitude)
            let loc2 = CLLocation(latitude: service.route.routeOptions.waypoints.last!.coordinate.latitude, longitude: service.route.routeOptions.waypoints.last!.coordinate.longitude)
            let distance = loc1.distance(from: loc2)
            if distance < 10 {
                if runTime_seconds > 180 {
                    stopMyNavigation()
                }
            }
        }
        return true
    }
    
    public func navigationService(_ service: NavigationService, didUpdate progress: RouteProgress, with location: CLLocation, rawLocation: CLLocation) {
        var value:Float = 0.0
        var valueImage:Float = 0.0
        if let lastDistanceRemaining = self.lastDistanceRemaining
        {
            secondaryInstruction_lbl.text = "\(lastDistanceRemaining - progress.distanceRemaining)"
            if lastDistanceRemaining - progress.distanceRemaining >= 25 ||   lastDistanceRemaining - progress.distanceRemaining <= -25
            {
                if  lastDistanceRemaining - progress.distanceRemaining <= -25
                {
                    playSynthesizerDirectionnal(description: languageRessource.wrongWay)
                }else if lastDistanceRemaining - progress.distanceRemaining >= 25
                {
                    if location.horizontalAccuracy > 20
                    {
                        if !navigationService.router.userIsOnRoute(location)
                        {
                            playSynthesizerDirectionnal(description: languageRessource.wrongWay)
                        }
                    }
                }
                self.lastDistanceRemaining = progress.distanceRemaining
            }
        }
        if  self.lastDistanceRemaining == nil && navigationService.route != wayToRoute
        {
            self.lastDistanceRemaining = progress.distanceRemaining
        }
        if let distance1 = distanceWayToRoute , let distance2 = distanceRoute
        {
            if progress.route == wayToRoute
            {
                value = Float(progress.distanceTraveled/(distance1+distance2))
                valueImage = Float(containerProgress_view.frame.width) * Float(value)
               
            } else if progress.route == myRoute
            {
                value = Float((progress.distanceTraveled+distance1)/(distance1+distance2))
                valueImage = Float(containerProgress_view.frame.width) * Float(value)
            }
        UIView.animate(withDuration: 0.5) {
            self.progressRoute_ProgressBar.setProgress(Float(value), animated: true)
            
            self.progressRouteIcon_imageView.center = CGPoint.init(x: Int(valueImage), y:   Int(self.progressRouteIcon_imageView.center.y))
            
        }
        
        if(self.previousLocation != nil && wayToRoute == nil)
        {
            
            if rawLocation != self.previousLocation
            {
                self.detectPOI(in: rawLocation, from: self.previousLocation)
            }
            
        }
        
        if (wayToRoute != nil)
        {
            if location.horizontalAccuracy  < 30
            {
                if let coordinates = myRoute?.coordinates
                {
                    for coordinate in coordinates {
                        if location.distance(from: CLLocation.init(latitude: coordinate.latitude, longitude: coordinate.longitude)) < 30 , self.calculatingRoute == false
                        {
                            self.calculatingRoute = true
                            stopNavigation()
                            let myLocation = LocationService.shared.getCurrentLocation()
                            if routeInformation?.isStartingPointStatic == false
                            {
                                routeCoordinates = MyUtils.changeBeginningOfSmartRunWithnearestPoint(routeCoordinates!, positionGPS: myLocation)
                            }
                            if let myCoordinate = myLocation?.coordinate
                            {
                                routeCoordinates?.insert(myCoordinate, at: 0)
                                if routeCoordinates!.count >= 100
                                {
                                    routeCoordinates!.removeLast()
                                }
                            }
                            DispatchQueue.main.asyncAfter(deadline: DispatchTime.now()+0.4) {
                                if let routeInformation = self.routeInformation 
                                {
                                    PlayerManager.shared.playSynt(description: self.languageRessource.routeReached)
                                    if routeInformation.profileIdentifier == 0
                                    {
                                        
                                        self.presenter.calculateRoute(points: self.routeCoordinates!, locale: self.languageAndCountry,measurementSystem: self.measurementSystem, objectFor: self, profileIdentifier: MBDirectionsProfileIdentifier.walking, firstCalculation: false)
                                    } else if routeInformation.profileIdentifier == 1
                                    {
                                        
                                        self.presenter.calculateRoute(points: self.routeCoordinates!, locale: self.languageAndCountry,measurementSystem: self.measurementSystem, objectFor: self, profileIdentifier: MBDirectionsProfileIdentifier.cycling, firstCalculation: false)
                                    }
                                }
                            }
                            
                        }
                    }
                }
            }
        }
        
        if   self.previousLocation != rawLocation
        {
            self.previousLocation = rawLocation
        }
    }
    
    
    }
 

}



extension MyNavigationView
{
    
    
    @objc func timerDuringSmartRun(){
        runTime_seconds          += 1
        var minute:     Int?
        var seconde:    Int?
        var heure:      Int?
        heure       = runTime_seconds/3600
        minute      = (runTime_seconds % 3600) / 60
        seconde     = (runTime_seconds % 3600 ) % 60
        if minute != nil && seconde != nil && heure != nil {
            DispatchQueue.main.async {
                self.timerValue_lbl.text =   String(format: "%02d", heure!) + ":" + String(format: "%02d", minute!) + ":" + String(format: "%02d", seconde!)
            }
        }
    }
    
    func initRouteInformations() -> Void {
        self.timerCourse = Timer.scheduledTimer(timeInterval: 1, target: self, selector: #selector(self.timerDuringSmartRun), userInfo: nil, repeats: true)
        //  self.timestamp = Int64(Date().timeIntervalSince1970)
        timerCourse.fire()
    }
    
    
    
    func checkIfInDirectionalZone(_ myDirectionPOIArray : Array<PointOfInterest>?, _ userPosition : CLLocation) -> Bool {
         guard userPosition.horizontalAccuracy < 50 else {
             return false
         }
         
         if let tempArrayPOIDirection = myDirectionPOIArray {
             for poiDirection in tempArrayPOIDirection {
                 for coorGPS in poiDirection.getLocations() {
                     let delta: CLLocationDistance = userPosition.distance(from: coorGPS)
                     if delta < Double(poiDirection.range) {
                         NSLog("is in Direction zone")
                         return true
                     }
                 }
             }
             return false
         }else{
             return false
         }
     }
     
     
     
     func giveDirectionFrom(previousLocation: CLLocation, actualLocation: CLLocation, poiLocation: CLLocation, resourceForTTS: LanguageRessource) -> String {
         let projection = projectOnNewReference(previousLocation, actualLocation, poiLocation)
         let distance = poiLocation.distance(from: projection.coordinates)
         if distance > 10 {
             if projection.parentCoordinatesInNewReference.coordinate.longitude > 0 {
                 return resourceForTTS.toYourRight
             }else{
                 return resourceForTTS.toYourLeft
             }
         } else {
             return resourceForTTS.inFront
         }
     }
    
    func detectPOI(in actualLocation : CLLocation, from previousLocation : CLLocation?) {

        if actualLocation.horizontalAccuracy < 50 { //We add 5 seconds to POI detector in order to let the TTS to annouce the user arrived on starting point without
            MyNavigationView.mutedZone = checkIfInDirectionalZone(directionals, actualLocation)
            
            if PlayerManager.shared.arrayStringPOIQueue.count > 0
            {
                if PlayerManager.shared.speechSynthesizer.isSpeaking == false &&  self.voiceController?.audioPlayer?.isPlaying == false
                {
                    PlayerManager.shared.speechSynthesizer.continueSpeaking()
                }
            }
            for (index,directional) in directionals.enumerated()
                
            {
                for (_, location ) in directional.getLocations().enumerated()
                {
                    let delta: CLLocationDistance = actualLocation.distance(from: location)
                    if delta < Double(directional.range) {
                        if (directionals[index].status == .notPlayed && directionals[index].isBloquant == false)
                        {
                            directionals[index].status = .isPlaying
                            if ( PlayerManager.shared.speechSynthesizer.isSpeaking)
                            {
                                PlayerManager.shared.speechSynthesizer.pauseSpeaking(at: .word)
                               
                            }
                            self.primaryInstruction_lbl.text = directional.description
                            playSynthesizerDirectionnal(description: directional.description!)
                        }
                        else if (directionals[index].status == .notPlayed && directionals[index].isBloquant == false)
                        {
                        }
                    }
                }
            }
            for (index,poi) in pois!.enumerated() {
                for (location ) in poi.getLocations()
                {
                    let delta: CLLocationDistance = actualLocation.distance(from: location)
                    if delta < Double(poi.range) {
                        if (pois![index].status == .notPlayed)
                        {
                            Locale.nationalizedCurrent = Locale.init(identifier: languageAndCountry)
                            pois![index].status = .isPlaying
                            AudioServicesPlayAlertSoundWithCompletion(SystemSoundID(kSystemSoundID_Vibrate)) { }
                            
                            if let player = self.voiceController?.audioPlayer
                            {
                                if player.isPlaying
                                {
                                    let time = player.duration-player.currentTime + 2
                                }
                                else
                                {
                                    
                                    
                                    let speakString: String = giveDirectionFrom(previousLocation: previousLocation!, actualLocation: actualLocation, poiLocation: location,resourceForTTS: languageRessource) +  poi.description!
                                    PlayerManager.shared.playSynt(description: speakString)
                                    pois![index].status = .isPlaying
                                }
                            }
                            else
                            {
                               
                                let speakString: String = giveDirectionFrom(previousLocation: previousLocation!, actualLocation: actualLocation, poiLocation: location,resourceForTTS: languageRessource) +  poi.description!
                                PlayerManager.shared.playSynt(description: speakString)
                                pois![index].status = .isPlaying
                                
                            }
                        }
                    }
                }
            }
        }
    }

    
    

    func projectOnNewReference(_ previousLocation: CLLocation, _ nextLocation: CLLocation, _ poiLocation: CLLocation) -> ProjectionUtils {
        
        //All coordinates from CLLocation are written with reference 0
        //All coordinates projected in OA vector abscisse are written with reference 1
        
        var xO0 = previousLocation.coordinate.latitude
        var yO0 = previousLocation.coordinate.longitude
        var xA0 = nextLocation.coordinate.latitude
        var yA0 = nextLocation.coordinate.longitude
        var xB0 = poiLocation.coordinate.latitude
        var yB0 = poiLocation.coordinate.longitude
        
        
        //On déplace le repère
        xA0 = xA0-xO0
        yA0 = yA0-yO0
        xB0 = xB0-xO0
        yB0 = yB0-yO0
        xO0 = 0
        yO0 = 0
        
        
        let OA = sqrt(pow(Double(xA0),2) + pow(Double(yA0),2))
        let cosAlpha  = xA0 / OA
        let sinAlpha  = yA0 / OA
        let tanAlpha    = sinAlpha/cosAlpha
        
        // let Alpha = atan(tanAlpha)
        
        //Le changement de repère donne y1= cos(Alpha)y0 - sin(Alpha)x0
        //Le changement de repère donne x1= cos(Alpha)x0 + sin(Alpha)y0
        
        let xA1 =  cosAlpha*xA0 + sinAlpha*yA0
        // let yA1 = cosAlpha*yA0 - sinAlpha*xA0
        let xB1 = cosAlpha*xB0 + sinAlpha*yB0
        let yB1 = (-sinAlpha*xB0) + (cosAlpha*yB0)
        
        var isOnPlot = false
        if abs(xB1) < abs(xA1) {
            //xB is either in OA or before O.
            
            if xB1 > xO0 {
                isOnPlot = true
            }else{
                isOnPlot = false
            }
        }
        
        let projection = ProjectionUtils.init()
        projection.isOnPlot = isOnPlot
        
        //Projection of B onto OA. In 0 reference
        let xP0 = cosAlpha*xB1 + (previousLocation.coordinate.latitude)
        let yP0 = sinAlpha*xB1 + (previousLocation.coordinate.longitude)
        
        let coordinates = CLLocation(latitude: xP0, longitude: yP0)
        let parentCoordinatesInNewReference = CLLocation(latitude: xB1, longitude: yB1)
        
        projection.coordinates = coordinates
        projection.parentCoordinatesInNewReference = parentCoordinatesInNewReference
        
        
        return projection
    }
}

==> v4_navigation_engine-0.1.31.tar.gz: ios/Classes/NavigationV4/View/MyNavigationRouteView+NavigationServiceDelegate.swift <==
//
//  MyNavigationRouteView+NavigationServiceDelegate.swift
//  NavigationUI
//
//  Created by hassine othmane on 4/2/20.
//  Copyright © 2020 com.mile. All rights reserved.
//

import MapboxCoreNavigation
import MapboxDirections
import CoreLocation
import MapboxNavigation


import MapboxSpeech
import AVKit

extension MyNavigationView :NavigationServiceDelegate {
    
    public func navigationService(_ service: NavigationService, shouldRerouteFrom location: CLLocation) -> Bool {
        if service.route == wayToRoute {
            return true
        }
        return false
    }

    

    public func navigationService(_ service: NavigationService, didArriveAt waypoint: Waypoint) -> Bool {
        
        if wayToRoute != nil
        {
            stopNavigation()
            PlayerManager.shared.playSynt(description: self.languageRessource.routeReached)
            DispatchQueue.main.asyncAfter(deadline: DispatchTime.now()+2) {
                self.initNavigation(userRoute: self.myRoute!)
            }
        }
        else
        {
            // you must check waypoints returned by calculate route
            let loc1 = CLLocation(latitude: waypoint.coordinate.latitude, longitude: waypoint.coordinate.longitude)
            let loc2 = CLLocation(latitude: service.route.routeOptions.waypoints.last!.coordinate.latitude, longitude: service.route.routeOptions.waypoints.last!.coordinate.longitude)
            let distance = loc1.distance(from: loc2)
            if distance < 10 { //if i'm at 10 meter of end point
                if runTime_seconds > 180 { // and stop my navigation if 3 minutes of run
                    stopMyNavigation()
                }
            }
        }
        return true
    }
    
    public func navigationService(_ service: NavigationService, didUpdate progress: RouteProgress, with location: CLLocation, rawLocation: CLLocation) {
        var value:Float = 0.0
        var valueImage:Float = 0.0
        if let lastDistanceRemaining = self.lastDistanceRemaining
        {
            secondaryInstruction_lbl.text = "\(lastDistanceRemaining - progress.distanceRemaining)"
            if lastDistanceRemaining - progress.distanceRemaining >= 25 ||   lastDistanceRemaining - progress.distanceRemaining <= -25
            {
                if  lastDistanceRemaining - progress.distanceRemaining <= -25
                {
                    playSynthesizerDirectionnal(description: languageRessource.wrongWay)
                }else if lastDistanceRemaining - progress.distanceRemaining >= 25
                {
                    if location.horizontalAccuracy < 20
                    {
                        if !navigationService.router.userIsOnRoute(location)
                        {
                            playSynthesizerDirectionnal(description: languageRessource.wrongWay)
                        }
                    }
                }
                self.lastDistanceRemaining = progress.distanceRemaining
            }
        }
        if  self.lastDistanceRemaining == nil && navigationService.route != wayToRoute
        {
            self.lastDistanceRemaining = progress.distanceRemaining
        }
        if let distance1 = distanceWayToRoute , let distance2 = distanceRoute
        {
            if progress.route == wayToRoute
            {
                value = Float(progress.distanceTraveled/(distance1+distance2))
                valueImage = Float(containerProgress_view.frame.width) * Float(value)
               
            } else if progress.route == myRoute
            {
                value = Float((progress.distanceTraveled+distance1)/(distance1+distance2))
                valueImage = Float(containerProgress_view.frame.width) * Float(value)
            }
        UIView.animate(withDuration: 0.5) {
            self.progressRoute_ProgressBar.setProgress(Float(value), animated: true)
            
            self.progressRouteIcon_imageView.center = CGPoint.init(x: Int(valueImage), y:   Int(self.progressRouteIcon_imageView.center.y))
            
        }
        
        if(self.previousLocation != nil && wayToRoute == nil)
        {
            
            if rawLocation != self.previousLocation
            {
                self.detectPOI(in: rawLocation, from: self.previousLocation)
            }
            
        }
        
        if (wayToRoute != nil)
        {
            if location.horizontalAccuracy  < 30
            {
                if let coordinates = myRoute?.coordinates
                {
                    for coordinate in coordinates {
                        if location.distance(from: CLLocation.init(latitude: coordinate.latitude, longitude: coordinate.longitude)) < 30 , self.calculatingRoute == false
                        {
                            self.calculatingRoute = true
                            stopNavigation()
                            let myLocation = LocationService.shared.getCurrentLocation()
                            if routeInformation?.isStartingPointStatic == false
                            {
                                routeCoordinates = MyUtils.changeBeginningOfSmartRunWithnearestPoint(routeCoordinates!, positionGPS: myLocation)
                            }
                            if let myCoordinate = myLocation?.coordinate
                            {
                                routeCoordinates?.insert(myCoordinate, at: 0)
                                if routeCoordinates!.count >= 100
                                {
                                    routeCoordinates!.removeLast()
                                }
                            }
                            DispatchQueue.main.asyncAfter(deadline: DispatchTime.now()+0.4) {
                                if let routeInformation = self.routeInformation 
                                {
                                    PlayerManager.shared.playSynt(description: self.languageRessource.routeReached)
                                    if routeInformation.profileIdentifier == "walking"
                                    {
                                        
                                        self.presenter.calculateRoute(points: self.routeCoordinates!, locale: self.languageAndCountry,measurementSystem: self.measurementSystem, objectFor: self, profileIdentifier: MBDirectionsProfileIdentifier.walking, firstCalculation: false)
                                    } else if routeInformation.profileIdentifier == "cycling" {
                                        
                                        self.presenter.calculateRoute(points: self.routeCoordinates!, locale: self.languageAndCountry,measurementSystem: self.measurementSystem, objectFor: self, profileIdentifier: MBDirectionsProfileIdentifier.cycling, firstCalculation: false)
                                    }
                                }
                            }
                            
                        }
                    }
                }
            }
        }
        
        if   self.previousLocation != rawLocation
        {
            self.previousLocation = rawLocation
        }
    }
    
    
    }
 

}



extension MyNavigationView
{
    
    
    @objc func timerDuringSmartRun(){
        runTime_seconds          += 1
        var minute:     Int?
        var seconde:    Int?
        var heure:      Int?
        heure       = runTime_seconds/3600
        minute      = (runTime_seconds % 3600) / 60
        seconde     = (runTime_seconds % 3600 ) % 60
        if minute != nil && seconde != nil && heure != nil {
            DispatchQueue.main.async {
                self.timerValue_lbl.text =   String(format: "%02d", heure!) + ":" + String(format: "%02d", minute!) + ":" + String(format: "%02d", seconde!)
            }
        }
    }
    
    func initRouteInformations() -> Void {
        self.timerCourse = Timer.scheduledTimer(timeInterval: 1, target: self, selector: #selector(self.timerDuringSmartRun), userInfo: nil, repeats: true)
        //  self.timestamp = Int64(Date().timeIntervalSince1970)
        timerCourse.fire()
    }
    
    
    
    func checkIfInDirectionalZone(_ myDirectionPOIArray : Array<PointOfInterest>?, _ userPosition : CLLocation) -> Bool {
         guard userPosition.horizontalAccuracy < 50 else {
             return false
         }
         
         if let tempArrayPOIDirection = myDirectionPOIArray {
             for poiDirection in tempArrayPOIDirection {
                 for coorGPS in poiDirection.getLocations() {
                     let delta: CLLocationDistance = userPosition.distance(from: coorGPS)
                     if delta < Double(poiDirection.range) {
                         NSLog("is in Direction zone")
                         return true
                     }
                 }
             }
             return false
         }else{
             return false
         }
     }
     
     
     
     func giveDirectionFrom(previousLocation: CLLocation, actualLocation: CLLocation, poiLocation: CLLocation, resourceForTTS: LanguageRessource) -> String {
         let projection = projectOnNewReference(previousLocation, actualLocation, poiLocation)
         let distance = poiLocation.distance(from: projection.coordinates)
         if distance > 10 {
             if projection.parentCoordinatesInNewReference.coordinate.longitude > 0 {
                 return resourceForTTS.toYourRight
             }else{
                 return resourceForTTS.toYourLeft
             }
         } else {
             return resourceForTTS.inFront
         }
     }
    
    func detectPOI(in actualLocation : CLLocation, from previousLocation : CLLocation?) {

        if actualLocation.horizontalAccuracy < 50 { //We add 5 seconds to POI detector in order to let the TTS to annouce the user arrived on starting point without
            MyNavigationView.mutedZone = checkIfInDirectionalZone(directionals, actualLocation)
            
            if PlayerManager.shared.arrayStringPOIQueue.count > 0 {
                if PlayerManager.shared.speechSynthesizer.isSpeaking == false && self.voiceController?.audioPlayer?.isPlaying == false {
                    PlayerManager.shared.speechSynthesizer.continueSpeaking()
                }
            }

            for (index,directional) in directionals.enumerated() {
                for (_, location ) in directional.getLocations().enumerated() {
                    let delta: CLLocationDistance = actualLocation.distance(from: location)
                    if delta < Double(directional.range) {
                        if (directionals[index].status == .notPlayed && directionals[index].isBloquant == false) {
                            directionals[index].status = .isPlaying
                            if ( PlayerManager.shared.speechSynthesizer.isSpeaking) {
                                PlayerManager.shared.speechSynthesizer.pauseSpeaking(at: .word)
                            }

                            self.primaryInstruction_lbl.text = directional.description
                            playSynthesizerDirectionnal(description: directional.description!)
                        }else if (directionals[index].status == .notPlayed && directionals[index].isBloquant == false) {

                        }
                    }
                }
            }

            for (index,poi) in pois!.enumerated() {
                for (location ) in poi.getLocations() {
                    let delta: CLLocationDistance = actualLocation.distance(from: location)
                    if delta < Double(poi.range) {
                        if (pois![index].status == .notPlayed) {
                            Locale.nationalizedCurrent = Locale.init(identifier: languageAndCountry)
                            pois![index].status = .isPlaying
                            AudioServicesPlayAlertSoundWithCompletion(SystemSoundID(kSystemSoundID_Vibrate)) { }
                            
                            if let player = self.voiceController?.audioPlayer {
                                if player.isPlaying {
                                    let time = player.duration-player.currentTime + 2
                                } else {
                                    let speakString: String = giveDirectionFrom(previousLocation: previousLocation!, actualLocation: actualLocation, poiLocation: location,resourceForTTS: languageRessource) +  poi.description!
                                    flutterOnActivePOI(poiId: pois![index].key, direction: "", play: true)
                                    //PlayerManager.shared.playSynt(description: speakString)
                                    pois![index].status = .isPlaying
                                }
                            } else {

                                let speakString: String = giveDirectionFrom(previousLocation: previousLocation!, actualLocation: actualLocation, poiLocation: location,resourceForTTS: languageRessource) +  poi.description!
                                flutterOnActivePOI(poiId: pois![index].key, direction: "", play: true)
                                //PlayerManager.shared.playSynt(description: speakString)
                                pois![index].status = .isPlaying
                                
                            }
                        }
                    }
                }
            }
        }
    }

    
    

    func projectOnNewReference(_ previousLocation: CLLocation, _ nextLocation: CLLocation, _ poiLocation: CLLocation) -> ProjectionUtils {
        
        //All coordinates from CLLocation are written with reference 0
        //All coordinates projected in OA vector abscisse are written with reference 1
        
        var xO0 = previousLocation.coordinate.latitude
        var yO0 = previousLocation.coordinate.longitude
        var xA0 = nextLocation.coordinate.latitude
        var yA0 = nextLocation.coordinate.longitude
        var xB0 = poiLocation.coordinate.latitude
        var yB0 = poiLocation.coordinate.longitude
        
        
        //On déplace le repère
        xA0 = xA0-xO0
        yA0 = yA0-yO0
        xB0 = xB0-xO0
        yB0 = yB0-yO0
        xO0 = 0
        yO0 = 0
        
        
        let OA = sqrt(pow(Double(xA0),2) + pow(Double(yA0),2))
        let cosAlpha  = xA0 / OA
        let sinAlpha  = yA0 / OA
        let tanAlpha    = sinAlpha/cosAlpha
        
        // let Alpha = atan(tanAlpha)
        
        //Le changement de repère donne y1= cos(Alpha)y0 - sin(Alpha)x0
        //Le changement de repère donne x1= cos(Alpha)x0 + sin(Alpha)y0
        
        let xA1 =  cosAlpha*xA0 + sinAlpha*yA0
        // let yA1 = cosAlpha*yA0 - sinAlpha*xA0
        let xB1 = cosAlpha*xB0 + sinAlpha*yB0
        let yB1 = (-sinAlpha*xB0) + (cosAlpha*yB0)
        
        var isOnPlot = false
        if abs(xB1) < abs(xA1) {
            //xB is either in OA or before O.
            
            if xB1 > xO0 {
                isOnPlot = true
            }else{
                isOnPlot = false
            }
        }
        
        let projection = ProjectionUtils.init()
        projection.isOnPlot = isOnPlot
        
        //Projection of B onto OA. In 0 reference
        let xP0 = cosAlpha*xB1 + (previousLocation.coordinate.latitude)
        let yP0 = sinAlpha*xB1 + (previousLocation.coordinate.longitude)
        
        let coordinates = CLLocation(latitude: xP0, longitude: yP0)
        let parentCoordinatesInNewReference = CLLocation(latitude: xB1, longitude: yB1)
        
        projection.coordinates = coordinates
        projection.parentCoordinatesInNewReference = parentCoordinatesInNewReference
        
        
        return projection
    }
}
